import os

def merge(old_dir,new_dir):
    old_res_dir = old_dir+"/res"
    new_res_dir = new_dir+"/res"
    # 自己也可以指定文件夹
    dirs = []
    # 获取res下所有的文件夹
    dirs.extend(os.listdir(old_res_dir))
    for dir in dirs:
        old_child_dir = old_res_dir+"/"+dir
        new_child_dir = new_res_dir+"/"+dir
        #如果不存在文件夹
        if(not os.path.isdir(new_child_dir)):
            os.system('mkdir {dir}'.format(dir=new_child_dir))
        #列出文件夹下所有文件
        l = list(map(lambda x:x[0:x.index('.')],os.listdir(new_child_dir)))
        print(l)
        for file in os.listdir(old_child_dir):
            #如果文件不存在
            file_name = file[0:file.index('.')]
            if(file_name not in l):
                layout_file = new_res_dir + "/layout"+file
                if 'layout-' in dir and os.path.isfile(layout_file):
                    pass
                else:
                    cp = 'cp {from_dir}/{file} {to_dir}'.format(from_dir=old_child_dir,file=file,to_dir=new_child_dir).replace("$", "\$")
                    os.system(cp)
                    


from lxml import etree
import os
#获取所有以layout或者xml开头的文件夹

def merge(old_dir,new_dir):
    files = os.listdir(old_dir+"/res/values")
    for file in files:
        merge_one_file(old_dir,new_dir,file)
    
def merge_one_file(old_dir,new_dir,file):
    old_file = old_dir+"/res/values/"+file
    old_file = old_dir+"/res/{dir}/{file}"
    new_file = new_dir+"/res/{dir}/{file}"
    #获取所有以values开头的文件夹
    dirs = list(filter(lambda x:x.startswith('values'),os.listdir(old_dir+"/res/")))
    # key是name value是Element
    for dir in dirs:
        from_f = old_file.format(dir=dir,file=file)
        to_f = new_file.format(dir=dir,file=file)
        change = False
        if os.path.isfile(from_f) and os.path.isfile(to_f):
            dict = {}
            to_tree = etree.parse(to_f)
            for child in to_tree.getroot():
                name = child.attrib['name']
                dict[name] = child
            from_tree = etree.parse(from_f)
            for child in from_tree.getroot():
                name = child.attrib['name'] 
                if name not in dict and "APKTOOL" not in name:
                    change = True
                    to_tree.getroot().append(child)

            if change:
                xml = '<?xml version="1.0" encoding="utf-8"?>\n'
                xml +='<resources>\n'
                for child in to_tree.getroot():
                    tag = child.tag
                    text = child.text
                    xml += '    '
                    # >会被转义
                    if(tag == 'string' and '>' in str(text)):
                        xml += etree.tostring(child,encoding='utf-8',pretty_print=True).decode('utf-8').strip().replace('/>',' />').replace('&gt;','>')
                    else:
                        xml += etree.tostring(child,encoding='utf-8',pretty_print=True).decode('utf-8').strip().replace('/>',' />')
                    xml +='\n'
                xml +='</resources>'
                with open(to_f,'w') as f:
                    f.write(xml)



from lxml import etree


#用于存放每种类型的最大值key是类型value是最大值
max_dict = {}
#key是类型value是name的集合
name_dict= {}


def merge(old_dir,new_dir):
    old_public = old_dir+"/res/values/public.xml"
    new_public = new_dir+"/res/values/public.xml"
    #先解析新的public.xml
    new_tree = etree.parse(new_public)
    for child in new_tree.getroot():
        if(child.tag=="public" and "id" in child.attrib):
            name=child.attrib['name']
            type=child.attrib['type']
            id=child.attrib['id']
            if(type in name_dict):
                pass
            else:
                name_dict[type] = []
            name_dict[type].append(name)
            #十六进制字符串转换为转十进制的int类型
            id = int(id,16) 
            if(type in max_dict):
                if(id > int(max_dict[type].attrib['id'],16)):
                    max_dict[type] = child
            else:
                max_dict[type] = child
    old_tree = etree.parse(old_public)
    for child in old_tree.getroot():
        if(child.tag=="public" and "id" in child.attrib):
            name=child.attrib['name']
            type=child.attrib['type']
             #屏蔽APKTOOL开头的name
            if(name not in name_dict[type] and "APKTOOL" not in name):
                max = max_dict[type]
                child.set('id',str(hex(int(max.attrib['id'],16)+1)))
                max_dict[type] = child
                index=new_tree.getroot().index(max)
                new_tree.getroot().insert(index+1,child)
    #不使用ET.write()方法，因为格式和转义
    #to_tree.write(to_public,encoding="utf-8",method="xml",pretty_print=True,xml_declaration=True,with_tail=True,strip_text=False)
    xml = '<?xml version="1.0" encoding="utf-8"?>\n'
    xml += '<resources>\n'
    for child in new_tree.getroot():
        name=child.attrib['name']
        type=child.attrib['type']
        id=child.attrib['id']
        xml += '    <public type="{type}" name="{name}" id="{id}" />'.format(type=type,name=name,id=id)
        xml += '\n'
    xml += '</resources>'
    with open(new_public,'w') as f:
        f.write(xml)



#该脚本用于合并资源
import argparse
from  merge_res.merge_file import merge_file
from  merge_res.merge_xml import merge_xml
from  merge_res.merge_public import merge_public

   

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("old_dir")
    parser.add_argument("new_dir")
    parser.add_argument("option")
    args = parser.parse_args()
    old_dir = args.old_dir
    new_dir = args.new_dir
    if args.option == 'all' or args.option is None:
        merge_public.merge(old_dir,new_dir)
        merge_file.merge(old_dir,new_dir)
        merge_xml.merge(old_dir,new_dir)
    elif args.option == 'public':
        merge_public.merge(old_dir,new_dir) 
    elif args.option == 'xml':
        merge_xml.merge(old_dir,new_dir) 
    elif args.option == 'file':
        merge_file.merge(old_dir,new_dir)
        
        
if __name__ == '__main__':
    main()


from setuptools import find_packages, setup

setup(
  name="merge_res",
  version="0.0.5",
  author="malinkang",
  author_email="linkang.ma@gmail.com",
  description="",
  long_description="",
  url="https://malinkang.com",
  packages=find_packages(),
    entry_points={
        "console_scripts": ["merge_res = merge_res.cli:main"],
    },
)